<?php

class Meow_SCLEGN_Core
{
	public $admin = null;
	public $is_rest = false;
	public $is_cli = false;
	public $site_url = null;
	public $services = array();
	protected $social_post_type = 'social_post';

	public function __construct() {
		$this->site_url = get_site_url();
		$this->is_rest = MeowCommon_Helpers::is_rest();
		$this->is_cli = defined( 'WP_CLI' ) && WP_CLI;

		// Actions
		add_action( 'plugins_loaded', array( $this, 'plugins_loaded' ) );
		add_action( 'init', array( $this, 'init') );
		add_action( 'transition_post_status', array( $this, 'transition_post_status' ), 10, 3 );

		// Services
		new Meow_SCLEGN_Services_Twitter();
		new Meow_SCLEGN_Services_LinkedIn();
		new Meow_SCLEGN_Services_Pinterest();

		if ( class_exists( 'MeowPro_SCLEGN_Core' ) ) {
			new MeowPro_SCLEGN_Core( $this );
		}
	}

	public function plugins_loaded() {
		// Part of the core, settings and stuff
		$this->admin = new Meow_SCLEGN_Admin();

		// Only for REST
		if ( $this->is_rest ) {
			new Meow_SCLEGN_Rest( $this, $this->admin );
		}

		// Dashboard
		if ( is_admin() ) {
			new Meow_SCLEGN_UI( $this, $this->admin );
		}

		global $sclegn;
		$sclegn = $this;
	}


	public function get_services() {
		return get_option( 'sclegn_services', array() );
	}

	public function get_modules() {
		return apply_filters( 'sclegn_modules', array() );
	}

	public function get_accounts() {
		return apply_filters( 'sclegn_accounts', array() );
	}

	public function get_service_settings( $id ) {
		$services = $this->get_services();
		foreach ( $services as $service ) {
			if ( $service['id'] === $id ) {
				return $service;
			}
		}
		return null;
	}

	public function get_service_instance( $type ) {
		$services = $this->get_modules();
		foreach ( $services as $service ) {
			if ( $service['type'] === $type ) {
				return $service['instance'];
			}
		}
		return null;
	}

	function transition_post_status( $newStatus, $oldStatus, $post ) {
		//error_log( "TRANSITION {$post->ID}: {$oldStatus} => {$newStatus}" );
		if ( $oldStatus !== 'publish' && $oldStatus !== 'new' && $newStatus === 'publish'
			&& $post->post_type === $this->social_post_type ) {
			//error_log( 'FUTURE TO PUBLISH', $post->ID );
			$this->post( $post->ID );
		}
	}

	public function request_service( $settings, $action, $params ) {
		$instance = $this->get_service_instance( $settings['type'] );
		return $instance->request_service( $settings, $action, $params );
	}

	public function post( $post_id ) {
		try {
			// Get the social post.
			$post = get_post( $post_id );
			$thumbnail_ids = json_decode( get_post_meta( $post->ID, 'sclegn_thumbnails', true ) ) ?? [];
			$thumbnail_paths = [];
			$thumbnail_urls = [];
			foreach ( $thumbnail_ids as $thumbnail_id ) {
				$thumbnail_path = realpath( get_attached_file( $thumbnail_id, true ) );
				$thumbnail_url = wp_get_attachment_image_url( $thumbnail_id, 'large' );
				if (!$thumbnail_path || !$thumbnail_url) {
					throw new Error( 'Can not find the attached image. Please check the image is exists.' );
				}
				$thumbnail_paths[] = $thumbnail_path;
				$thumbnail_urls[] = $thumbnail_url;
			}
			$social_post = [
				'id' => $post->ID,
				'post_content' => $post->post_content,
				'thumbnail_ids' => $thumbnail_ids,
				'thumbnail_paths' => $thumbnail_paths,
				'thumbnail_urls' => $thumbnail_urls,
				'post_date' => $post->post_date,
				'service_type' => get_post_meta( $post->ID, 'sclegn_service_type', true ),
				'account_id' => get_post_meta( $post->ID, 'sclegn_account_id', true ),
				'service_id' => get_post_meta( $post->ID, 'sclegn_service_id', true ),
				'post_status' => $post->post_status,
			];

			// Publish the social post.
			$settings = $this->get_service_settings( $social_post['service_id'] );
			// The service type of the social post is the priority when the difference between the social post and settings.
			$type = $social_post['service_type'] !== $settings['type'] ? $social_post['service_type'] : $settings['type'];
			$instance = $this->get_service_instance( $type );
			$result = null;

			try {
				$result = $instance->post( $social_post, $settings );
			}
			catch (Exception $e) {
				// TODO: We should do a better error handling here.
				wp_update_post( [ 'ID' => $social_post['id'], 'post_status' => 'draft' ], false, false );
				throw $e;
			}

			// Store the ID returned by the service posted to.
			update_post_meta( $post_id, 'sclegn_posted_id', $result['posted_id'] );

			// Update post
			$postDate = new DateTime( $result['post_date'] );
			$post_title = 'Published on ' . $postDate->format( 'Y/m/d' ) . ' at ' . 
				$postDate->format( 'H:i:s' ) . ' for ' . ucfirst( $social_post['service_type'] );
			$post_date = $result['post_date'];
			$update_post = [
				'ID' => $social_post['id'],
				'post_title' => $post_title,
				//'post_status' => 'publish',
				'post_date' => $post_date,
				'post_date_gmt' => get_gmt_from_date( $post_date ),
			];
			$post_id = wp_update_post( $update_post, true );
			if ( is_wp_error( $post_id ) ) {
				throw new Error( implode( ',', $post_id->get_error_messages() ) );
			}
		}
		catch ( \Exception $e ) {
			throw $e;
		}
	}

	public function post_stats( $post_id ) {
		$service_type = get_post_meta( $post_id, 'sclegn_service_type', true );
		$service_id = get_post_meta( $post_id, 'sclegn_service_id', true );
		$posted_id = get_post_meta( $post_id, 'sclegn_posted_id', true );
		$account_id = get_post_meta( $post_id, 'sclegn_account_id', true );
		$settings = $this->get_service_settings( $service_id );

		// The service type of the social post is the priority when the difference between the social post and settings.
		$type = $service_type !== $settings['type'] ? $service_type : $settings['type'];
		$instance = $this->get_service_instance( $type );

		try {
			return $instance->stats( $posted_id, $settings, $account_id );
		}
		catch (Exception $e) {
			throw $e;
		}
	}

	public function init() {
		register_post_type( 'social_post',
			[
				'labels' => [
					'name'          => __( 'Social Posts', 'social_engine' ),
					'singular_name' => __( 'Social Post', 'social_engine' ),
				],
				'supports' => [
					'editor',
					'thumbnail',
					'custom-fields',
				],
				'public' => false,
				'show_in_rest' => false
			]
		);
	}

	public function create_social_post( $date, $content, $status, $thumbnail_ids, $service_id, $account_id, $service_type ) {
		$post_date = $date->format( 'Y-m-d H:i:s' );

		$post = [
			'post_title' => 'Scheduled on ' . $date->format( 'Y/m/d' ) . ' at ' . $date->format( 'H:i:s' ) . ' for ' . ucfirst( $service_type ),
			'post_content' => $content,
			'post_date' => $post_date,
			'post_date_gmt' => get_gmt_from_date( $post_date ),
			'post_status' => $status,
			'post_type' => $this->social_post_type,
			'comment_status' => 'closed',
			'ping_status' => 'closed'
		];
		$post_id = wp_insert_post( $post, true );
		if ( is_wp_error( $post_id ) ) {
			throw new Error( implode( ',', $post_id->get_error_messages() ) );
		}

		// Add the thumbnails
		if ( !empty( $thumbnail_ids ) && count( $thumbnail_ids ) ) {
			$formatted_thumbnail_ids = [];
			foreach ( $thumbnail_ids as $thumbnail_id ) {
				$formatted_thumbnail_ids[] = (string) $thumbnail_id;
			}
			update_post_meta( $post_id, 'sclegn_thumbnails', json_encode($formatted_thumbnail_ids) );
		}

		// Update the metadata
		update_post_meta( $post_id, 'sclegn_service_id', $service_id );
		update_post_meta( $post_id, 'sclegn_account_id', $account_id );
		update_post_meta( $post_id, 'sclegn_service_type', $service_type );

		return $post_id;
	}

	public function update_social_post( $id, $date, $content, $status, $thumbnail_ids, $service_id, $account_id, $service_type ) {
		$post_date = $date->format( 'Y-m-d H:i:s' );
		$post = [
			'ID' => $id,
			'post_title' => 'Scheduled on ' . $date->format( 'Y/m/d' ) . ' at ' . $date->format( 'H:i:s' ) . ' for ' 
				. ucfirst( $service_type ),
			'post_date_gmt' => get_gmt_from_date( $post_date ),
			'post_type' => $this->social_post_type,
			'comment_status' => 'closed',
			'ping_status' => 'closed'
		];

		if ( !empty( $content ) ) {
			$post['post_content'] = $content;
		}
		if ( !empty( $post_date ) ) {
			$post['post_date'] = $post_date;
		}
		if ( !empty( $status ) ) {
			$post['post_status'] = $status;
		}

		$post_id = wp_update_post( $post, true );
		if ( is_wp_error( $post_id ) ) {
			throw new Error( implode( ',', $post_id->get_error_messages() ) );
		}

		// Update the thumbnails
		if ( count($thumbnail_ids) ) {
			$formatted_thumbnail_ids = [];
			foreach ( $thumbnail_ids as $thumbnail_id ) {
				$formatted_thumbnail_ids[] = (string) $thumbnail_id;
			}
			update_post_meta( $post_id, 'sclegn_thumbnails', json_encode($formatted_thumbnail_ids) );
		} else {
			delete_post_meta( $post_id, 'sclegn_thumbnails' );
		}

		// Update the metadata
		update_post_meta( $post_id, 'sclegn_service_id', $service_id );
		update_post_meta( $post_id, 'sclegn_account_id', $account_id );
		update_post_meta( $post_id, 'sclegn_service_type', $service_type );
	}

	public function delete_social_posts($ids) {
		foreach ( $ids as $id ) {
			$result = wp_delete_post( $id );
			if (!$result) {
				throw new Error('Failed to delete the social post.');
			}
		}
	}

	public function get_social_post_type() {
		return $this->social_post_type;
	}

	public function get_recent_photos($limit = 3, $search = null, $except_post_ids = []) {
		global $wpdb;
		$where_search_clause = $search ? $wpdb->prepare("AND p.post_content LIKE %s", '%' . $search . '%') : '';

		$where_except_post_ids_clause = '';
		if (count($except_post_ids)) {
			$post_ids_placeholders = implode(', ', array_fill(0, count( $except_post_ids ), '%s'));
			$where_except_post_ids_clause = $wpdb->prepare("AND p.ID NOT IN (" . $post_ids_placeholders . ")", $except_post_ids);
		}

		return $wpdb->get_results(
			$wpdb->prepare(
				"SELECT p.ID, pm1.meta_value 
				FROM $wpdb->posts p 
				INNER JOIN $wpdb->postmeta pm1 ON pm1.post_id = p.ID AND pm1.meta_key = 'sclegn_thumbnails' 
				WHERE p.post_type='social_post' 
				$where_search_clause 
				$where_except_post_ids_clause 
				ORDER BY p.post_modified DESC 
				LIMIT %d, %d", 0, $limit
			), ARRAY_A
		);
	}

	/**
	 *
	 * Roles & Access Rights
	 *
	 */
	public function can_access_settings() {
		$manage_options = current_user_can( 'manage_options' );
		return apply_filters( 'sclegn_allow_setup', $manage_options );
	}

	public function can_access_features() {
		$editor = current_user_can( 'administrator' ) || current_user_can( 'editor' );
		return apply_filters( 'sclegn_allow_usage', $editor );
	}

	public function can_access_moderation_features() {
		$admin = current_user_can( 'administrator' );
		return apply_filters( 'sclegn_allow_moderation', $admin );
	}
}

?>