<?php
require_once( SCLEGN_PATH . '/classes/services/pinterest/client.php' );

class Meow_SCLEGN_Services_Pinterest
{
  public function __construct() {
    add_action( 'plugins_loaded', array( $this, 'plugins_loaded' ), 10 );
  }

  public function plugins_loaded() {

    add_filter( 'sclegn_modules', function( $modules ) {
      if ( !in_array( 'pinterest', $modules ) ) {
        $modules[] = [
          'type' => 'pinterest',
          'instance' => $this
        ];
      }
      return $modules;
    }, 10, 1 );

    add_filter( 'sclegn_accounts', function( $accounts ) {
      $services = get_option( 'sclegn_services', array() );
      foreach ( $services as &$service ) {
        if ( $service['type'] === 'pinterest' && isset( $service['boards'] ) ) {
          foreach ( $service['boards'] as $board ) {
            if ( !$board['enabled'] ) continue;
            $accounts[] = [
              'type' => 'pinterest',
              'name' => $service['name'] . '(' . $board['name'] . ')',
              'serviceId' => $service['id'],
              'accountId' => $board['id'],
              'instance' => $this,
            ];
          }
        }
      }
      return $accounts;
    }, 10, 1 );
  }

  private function create_connection( $settings ) {
    // App ID === Client ID
    // App Secret Key === Client Secret

    if ( empty( $settings['app_id'] ) ) {
      throw new Exception("The Client ID is missing.");
    }
    if ( empty( $settings['app_secret_key'] ) ) {
      throw new Exception("The Client Secret is missing.");
    }
    if ( empty( $settings['access_token'] ) ) {
      throw new Exception("The Access Token is missing.");
    }
    if ( empty( $settings['access_token_expires_at'] ) ) {
      throw new Exception("The Access Token Expires At is missing.");
    }
    if ( empty( $settings['refresh_token'] ) ) {
      throw new Exception("The Refresh Access Token is missing.");
    }
    if ( empty( $settings['refresh_token_expires_at'] ) ) {
      throw new Exception("The Refresh Token Expires At is missing.");
    }
    $connection = new PinterestClient( $settings['app_id'], $settings['app_secret_key'] );
    $connection->setAccessToken( $settings['access_token'], $settings['access_token_expires_at'], $settings['refresh_token'], $settings['refresh_token_expires_at'] );
    return $connection;
  }

  private function get_login_url( $settings ) {
    if ( empty( $settings['app_id'] ) ) {
      throw new Exception("The Client ID is missing.");
    }
    if ( empty( $settings['app_secret_key'] ) ) {
      throw new Exception("The Client Secret is missing.");
    }
    if ( empty( $settings['redirect_uri'] ) ) {
      throw new Exception("The Redirect Uri is missing.");
    }
    if ( empty( $settings['state'] ) ) {
      throw new Exception("The state is missing.");
    }
    $client = new PinterestClient( $settings['app_id'], $settings['app_secret_key'] );
    $client->setRedirectUri( $settings['redirect_uri'] );
    $scopes = [
      'boards:read',
      'boards:read_secret',
      'boards:write',
      'boards:write_secret',
      'pins:read',
      'pins:read_secret',
      'pins:write',
      'pins:write_secret',
    ];
    return [
      'url' => $client->getOAuthPageUrl( $scopes, $settings['state'] ),
    ];
  }

  private function get_access_token( $settings, $params ) {
    if ( empty( $settings['app_id'] ) ) {
      throw new Exception("The Client ID is missing.");
    }
    if ( empty( $settings['app_secret_key'] ) ) {
      throw new Exception("The Client Secret is missing.");
    }
    if ( empty( $settings['redirect_uri'] ) ) {
      throw new Exception("The Redirect Uri is missing.");
    }
    if ( empty( $settings['state'] ) ) {
      throw new Exception("The state is missing in Settings.");
    }
    if ( empty( $params['code'] ) ) {
      throw new Exception("The code is missing.");
    }
    if ( empty( $params['state'] ) ) {
      throw new Exception("The state is missing.");
    }
    if ( $settings['state'] !== $params['state'] ) {
      throw new Exception("Invalid access.");
    }
    $client = new PinterestClient( $settings['app_id'], $settings['app_secret_key'] );
    $client->setRedirectUri( $settings['redirect_uri'] );
    $access_token = $client->getAccessToken( $params['code'] );
    return [
      'access_token' => $access_token->getAccessToken(),
      'access_token_expires_at' => $access_token->getAccessTokenExpiresAt(),
      'refresh_token' => $access_token->getRefreshToken(),
      'refresh_token_expires_at' => $access_token->getRefreshTokenExpiresAt(),
    ];
  }

  private function get_list_boards( $settings ) {
    $client = $this->create_connection( $settings );
    try {
      $response = $client->get( 'boards', [ 'page_size' => 25 ] );
      $list = [];
      foreach ( $response['items'] as $item ) {
        $list[] = [
          'id' => $item['id'],
          'name' => $item['name'],
          'enabled' => false,
        ];
      }
      return [
        'boards' => $list,
      ];
    } catch ( \Exception $exception ) {
      throw new Exception( $exception->getMessage(), $exception->getCode() );
    }
  }

  private function refresh_access_token( $settings ) {
    $client = $this->create_connection( $settings );
    $client->refreshAccessToken();
    $access_token = $client->getAccessToken();
    return [
      'access_token' => $access_token->getAccessToken(),
      'access_token_expires_at' => $access_token->getAccessTokenExpiresAt(),
    ];
  }

  // https://developers.pinterest.com/docs/api/v5/#operation/pins/create
  public function post( $social_post, $settings ) {
    $client = $this->create_connection( $settings );

    // Attach an image
    $media_source = [];
    if ( isset( $social_post['thumbnail_paths'] ) && count( $social_post['thumbnail_paths'] ) > 0 ) {
      $path = $social_post['thumbnail_paths'][0];
      $extension = pathinfo( $path, PATHINFO_EXTENSION );
      if ( $extension === 'jpeg' || $extension === 'jpg' ) {
        $extension = 'jpeg';
      }
      $media_source['media_source'] = [
        'source_type' => 'image_base64',
        'content_type' => 'image/' . $extension,
        'data' => base64_encode( file_get_contents( $path ) ),
      ];
    }

    // Body
    $body = array_merge( [
      'link' => null,
      'title' => null,
      'description' => $social_post['post_content'],
      'alt_text' => null,
      'board_id' => $social_post['account_id'],
      'board_section_id' => null,
    ], $media_source );

    try {
      $response = $client->post( 'pins', $body );
      return [
        'posted_id' => $response['id'],
        'post_date' => get_date_from_gmt( $response['created_at'], 'Y-m-d H:i:s' ),
      ];
    } catch ( \Exception $exception ) {
      throw new Exception( $exception->getMessage(), $exception->getCode() );
    }
  }

  public function request_service( $settings, $action, $params ) {
    if ( $action === "test" ) {
      $connection = $this->create_connection( $settings );
      // todo: implements if needed
    } elseif ( $action === "auth" ) {
      return $this->get_login_url( $settings );
    } elseif ( $action === "access-token" ) {
      return $this->get_access_token( $settings, $params );
    } elseif ( $action === "refresh-access-token" ) {
      return $this->refresh_access_token( $settings );
    } elseif ( $action === 'list-boards' ) {
      return $this->get_list_boards( $settings );
    }
    return false;
  }

  public function stats( $posted_id, $settings, $account_id ) {
    return [
      'message' => 'This is not supported for now.',
    ];
    // This feature needs to get "r_member_social" permission. (Marketing Developer Platform)
    // $connection = $this->create_connection( $settings );
    // try {
    //   $response = $connection->get('socialActions/urn:li:share:' . $posted_id . '/likes');
    // } catch ( \LinkedIn\Exception $exception ) {
    //   throw new Exception( $exception->getMessage(), $exception->getCode() );
    // }
    // return $response;
  }
}