<?php
require_once( SCLEGN_PATH . '/classes/services/pinterest/access_token.php' );

class PinterestClient {

    const OAUTH_PAGE_URL = 'https://www.pinterest.com/oauth/';
    const API_ROOT = 'https://api.pinterest.com/v5/';

    /**
     * Client Id
     *
     * @var string
     */
    protected $client_id;

    /**
     * Client Secret
     *
     * @var string
     */
    protected $client_secret;

    /**
     * Redirect URI
     *
     * @var string
     */
    protected $redirect_uri;

    /**
     * Access Token
     *
     * @var PinterestAccessToken
     */
    protected $access_token;

    /**
     * Constructor.
     *
     * @param string $client_id
     * @param string $client_secret
     */
    public function __construct( $client_id, $client_secret )
    {
        $this->client_id = $client_id;
        $this->client_secret = $client_secret;
    }

    /**
     * @param string $redirect_uri The URI your users will be sent back to after
     *                            authorization.  This value must match one of
     *                            the defined OAuth 2.0 Redirect URLs in your
     *                            application configuration.
     *
     * @return PinterestClient
     */
    public function setRedirectUri( $redirect_uri )
    {
        $redirect_uri = filter_var( $redirect_uri, FILTER_VALIDATE_URL );
        if ( $redirect_uri === false ) {
            throw new \InvalidArgumentException( 'The argument is not an URL' );
        }
        $this->redirect_uri = $redirect_uri;
        return $this;
    }

    /**
     * Get the OAuth page url to start the OAuth flow and request user access.
     *
     * @param array $scope
     * @param string $state
     */
    public function getOAuthPageUrl( $scope, $state )
    {
        $params = [
            'client_id' => $this->client_id,
            'redirect_uri' => $this->redirect_uri,
            'response_type' => 'code',
            'scope' => implode( ",", $scope ),
            'state' => $state,
        ];
        return self::OAUTH_PAGE_URL . '?' . $this->buildQuery( $params );
    }

    /**
     * Retrieve Access Token from Pinterest if we have code provided.
     * If code is not provided, return current Access Token.
     * If current access token is not set, will return null
     *
     * @param string $code
     *
     * @return PinterestAccessToken|null
     * @throws \Exception
     */
    public function getAccessToken( $code = '' )
    {
        if ( !empty( $code ) ) {
            $url = $this->buildUrl( 'oauth/token', [] );
            try {
                $args = [
                    'headers' => [
                        'Content-Type' => 'application/x-www-form-urlencoded',
                        'Authorization' => 'Basic ' . base64_encode( "{$this->client_id}:{$this->client_secret}" )
                    ],
                    'body' => [
                        'grant_type' => 'authorization_code',
                        'code' => $code,
                        'redirect_uri' => $this->redirect_uri,
                    ]
                ];
                $response = wp_remote_post( $url, $args );
                if ( is_wp_error( $response ) ) {
                    throw new \Exception( $response->get_error_message(), $response->get_error_code());
                }
            } catch ( \Exception $exception ) {
                throw $exception;
            }
            $response = $this->getBodyFromResponse( $response );
            $this->setAccessToken( PinterestAccessToken::fromResponse( $response ) );
        }
        return $this->access_token;
    }

    /**
     * Refresh Access Token by using refresh_token.
     *
     * @param string $code
     *
     * @return PinterestClient
     * @throws \Exception
     */
    public function refreshAccessToken()
    {
        $url = $this->buildUrl( 'oauth/token', [] );
        try {
            $args = [
                'headers' => [
                    'Content-Type' => 'application/x-www-form-urlencoded',
                    'Authorization' => 'Basic ' . base64_encode( "{$this->client_id}:{$this->client_secret}" )
                ],
                'body' => [
                    'grant_type' => 'refresh_token',
                    'refresh_token' => $this->access_token->getRefreshToken(),
                ]
            ];
            $response = wp_remote_post( $url, $args );
            if ( is_wp_error( $response ) ) {
                throw new \Exception( $response->get_error_message(), $response->get_error_code());
            }
        } catch ( \Exception $exception ) {
            throw $exception;
        }
        $response = $this->getBodyFromResponse( $response );
        $new_access_token = PinterestAccessToken::fromResponse( $response );
        $this->access_token->setAccessToken($new_access_token->getAccessToken());
        $this->access_token->setAccessTokenExpiresAt($new_access_token->getAccessTokenExpiresAt());
        return $this;
    }

    /**
     * Send get request to Pinterest.
     *
     * @param string $path
     * @param array $params
     *
     * @return PinterestAccessToken|null
     * @throws \Exception
     */
    public function get( $path, $params = [] )
    {
        $url = $this->buildUrl( $path, $params );
        try {
            $args = [
                'headers' => [
                    'Content-Type' => 'application/json',
                    'Authorization' => "Bearer {$this->access_token->getAccessToken()}"
                ],
            ];
            $response = wp_remote_get( $url, $args );
            if ( is_wp_error( $response ) ) {
                throw new \Exception( $response->get_error_message(), $response->get_error_code());
            }
        } catch ( \Exception $exception ) {
            throw $exception;
        }
        return $this->getBodyFromResponse( $response );
    }

    /**
     * Send post request to Pinterest.
     *
     * @param string $path
     * @param array $body
     *
     * @return PinterestAccessToken|null
     * @throws \Exception
     */
    public function post( $path, $body = [] )
    {
        $url = $this->buildUrl( $path, [] );
        try {
            $args = [
                'headers' => [
                    'Content-Type' => 'application/json',
                    'Authorization' => "Bearer {$this->access_token->getAccessToken()}",
                ],
                'body' => json_encode( $body ),
            ];
            $response = wp_remote_post( $url, $args );
            if ( is_wp_error( $response ) ) {
                throw new \Exception( $response->get_error_message(), $response->get_error_code());
            }
        } catch ( \Exception $exception ) {
            throw $exception;
        }
        return $this->getBodyFromResponse( $response );
    }


    /**
     * Set PinterestAccessToken object
     *
     * @param PinterestAccessToken|string $access_token
     * @param string $refresh_token
     *
     * @return PinterestClient
     */
    public function setAccessToken( $access_token, $access_token_expires_at = 0, $refresh_token = '', $refresh_token_expires_at = 0 )
    {
        if ( is_object( $access_token ) && $access_token instanceof PinterestAccessToken) {
            $this->access_token = $access_token;
        } elseif ( is_string( $access_token ) && is_int( $access_token_expires_at ) && is_string( $refresh_token ) && is_int( $refresh_token_expires_at ) ) {
            $this->access_token = new PinterestAccessToken( $access_token, $access_token_expires_at, $refresh_token, $refresh_token_expires_at );
        } else {
            throw new \InvalidArgumentException('$access_token must be instance of \PinterestAccessToken class');
        }
        return $this;
    }

    /**
     * @param string $endpoint
     * @param array  $params
     *
     * @return string
     */
    protected function buildUrl( $endpoint, $params )
    {
        $url = self::API_ROOT;
        $scheme = parse_url( $url, PHP_URL_SCHEME );
        $authority = parse_url( $url, PHP_URL_HOST );
        $path = parse_url( $url, PHP_URL_PATH );
        $path .= trim( $endpoint, '/' );
        $fragment = '';
        $query = $this->buildQuery( $params );

        $uri = '';
        if ( $scheme != '' ) {
            $uri .= $scheme . ':';
        }
        if ( $authority != ''|| $scheme === 'file' ) {
            $uri .= '//' . $authority;
        }
        $uri .= $path;

        if ( $query != '' ) {
            $uri .= '?' . $query;
        }
        if ( $fragment != '' ) {
            $uri .= '#' . $fragment;
        }
        return $uri;
    }

    /**
     * @param array $params
     *
     * @return string
     */
    protected function buildQuery( $params )
    {
        return http_build_query( $params, "", "&", PHP_QUERY_RFC3986 );
    }

    /**
     * Retrieve body from response
     *
     * @param array $response
     *
     * @return array|null
     */
    protected function getBodyFromResponse( $response )
    {
        $body = wp_remote_retrieve_body( $response );
        if ( $body ) {
            $data = json_decode( $body, true );
            if (JSON_ERROR_NONE !== json_last_error()) {
                throw new \InvalidArgumentException(
                    'json_decode error: ' . json_last_error_msg() . $data
                );
            }
            return $data;
        }
        return null;
    }

    /**
     * Retrieve body from response
     *
     * @param array $response
     *
     * @return array
     */
    protected function getHeaderFromResponse( $response )
    {
        $headers = wp_remote_retrieve_headers( $response );
        return $headers;
    }
}